/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvControlIf.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvSignalLine.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_RGiceTGU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5dsxYCf3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bWUqNhlj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_g1bEDdLn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0omsOB0k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_sHyHGXXL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MYFtLU8N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_B65W08ir_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NwfvYFWf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cRDKaZ9s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HnvMjIJQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_uvnZ6qpv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IF6p82ml_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_kqJFuZJr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_aVhA7nZB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_dgnCYS5x_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_4jDjQcto_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_QeiZ0fk4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_zXfkBk8w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_L3oJDtcf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XUAhBdzt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8Y2uYpYj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IFCbcw07_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_GNCazl9c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9yevGIDR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RARSxrL1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_RoaehAbK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6xD7qFqd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QyibBvRv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_m1xz2E66_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Jq8w1Qe3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_s6yeijEL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_glJEaC2I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_f7oOtpGw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_TnxtBXnt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_c9oHYDI4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7dyEPuYe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_n0bI5Byb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_lUkf7u0O_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xRDT1lET_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NuRxGaJm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SoAKbhex_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_g7Aiee7i_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_9fwLAA96_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_mcFY3Dey_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kj07x5D9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_gARJUXKC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KlHUEiDQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_6YiKOo3C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OVSKfVEr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uSfnIv2O_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KEktsgyp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_SfAvDFeD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_s9hoO8n4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xhaIPpgN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gRwGPzWL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pWi4Ttvi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_nNT5nvRP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_WT2FVRzD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Cjo8JxRx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LLNsfQld_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_jCSNpRDd_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_I9QKjQ6j;
  SignalEnvelope sEnv_QeiZ0fk4;
  SignalEnvelope sEnv_zXfkBk8w;
  SignalEnvelope sEnv_TnxtBXnt;
  SignalEnvelope sEnv_c9oHYDI4;
  SignalLine sLine_Zok161uS;
  SignalLine sLine_R4HnfERi;
  ControlBinop cBinop_RGiceTGU;
  ControlBinop cBinop_5dsxYCf3;
  ControlBinop cBinop_bWUqNhlj;
  ControlBinop cBinop_g1bEDdLn;
  ControlBinop cBinop_MYFtLU8N;
  ControlBinop cBinop_NwfvYFWf;
  ControlBinop cBinop_cRDKaZ9s;
  ControlBinop cBinop_HnvMjIJQ;
  ControlVar cVar_uvnZ6qpv;
  ControlVar cVar_kqJFuZJr;
  SignalVarf sVarf_GUU8SbU8;
  ControlVar cVar_aVhA7nZB;
  ControlIf cIf_dgnCYS5x;
  ControlIf cIf_4jDjQcto;
  ControlBinop cBinop_L3oJDtcf;
  ControlBinop cBinop_8Y2uYpYj;
  ControlBinop cBinop_IFCbcw07;
  ControlBinop cBinop_RARSxrL1;
  ControlBinop cBinop_6xD7qFqd;
  SignalVarf sVarf_7fUHCeha;
  ControlVar cVar_s6yeijEL;
  ControlIf cIf_glJEaC2I;
  ControlIf cIf_f7oOtpGw;
  ControlBinop cBinop_7dyEPuYe;
  ControlBinop cBinop_lUkf7u0O;
  ControlBinop cBinop_xRDT1lET;
  ControlBinop cBinop_g7Aiee7i;
  ControlBinop cBinop_mcFY3Dey;
  ControlVar cVar_6YiKOo3C;
  ControlBinop cBinop_OVSKfVEr;
  SignalVarf sVarf_usdy2v0m;
  SignalVarf sVarf_8JDa7FLc;
  SignalVarf sVarf_kBI2lmH4;
  SignalVarf sVarf_3oHd6RGR;
  ControlBinop cBinop_s9hoO8n4;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
